// ****************************** DYNATEST.CXX ***************************
// *  Test program for dynarray.
// *  Adapted from "Using C++" by Bruce Eckel, Osborne/McGraw-Hill 1989
// ***********************************************************************

#include "dynarray.hxx"
#include <stream.hxx>
#include <string.h>
#include <stdlib.h>

// Very simple class to save a string, and print
// it with streams:
class string
{
  char * str;

public:
  string(char * msg = "") { str = msg; }
  // create an automatic type conversion operator
  // so you don't have to cast string objects to
  // void pointer objects every time you want to
  // add them to a DynArray:
  operator void*() { return (void *)this; }
  char * cp() { return str; }
  friend ostream& operator<<(ostream &s, string * sp)
  {
    s << sp->str;
    return s;
  }
};

main()
{
  DynArray da;
  da.add(string("This is a ")); da.add(string("test of the "));
  da.add(string("dynamic array ")); da.add(string("class "));
  da.add(string("to \n")); da.add(string("(OOPS! A Mistake) "));
  da.add(string("see ")); da.add(string("if "));
  da.add(string("it ")); da.add(string("will "));
  da.add(string("automatically make "));
  da.add(string("itself bigger "));
  da.add(string("when it \n")); da.add(string("runs out of "));
  da.add(string("room."));
  // First, print the whole list:
  da.reset();
  do
    cout << (string *)da.current();
  while( da.next() );
  cout << "\n\n";

  // Now, find the element with the "OOPS":
  da.reset();
  while( strstr(((string *)da.current())->cp(), "OOPS") == 0 )
    if(da.next() == 0)
    {
      cerr << "OOPS not found\n";
      exit(1);
    }

  int rm = da.index(); // number of element to remove
  cout << "removing " << (string *)da[rm] << "\n";
  if (da.remove(rm))
    cout << "removed successfully!\n";

  // Print the whole list again:
  da.reset();
  do
    cout << (string *)da.current();
  while( da.next() );
}