/*********************** HOUSE1.C ************************
 * A demonstration of drawing scaling
 *
 * Author: John M. Dlugosz (port to Turbo C by John Rex)
 ********************************************************/

#include <graphics.h>
#include <math.h>
#include <conio.h>
#include <stdlib.h>

#define POINTMAX 12

typedef double real;

typedef struct {
	real x, y;
} realpoint;


real adjustment;   /* stretch the horizontal to proper aspect ratio */

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

/* set the adjustment variable */
void set_adjustment()
{
	int xasp,yasp;

	getaspectratio (&xasp,&yasp);
	adjustment = ((real) yasp) / ((real) xasp);
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void drawline(realpoint *source, realpoint *dest)
{
	line ((int) source->x, (int) source->y, (int) dest->x, (int) dest->y);
}


/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void drawrec (realpoint *ul, realpoint *size)
{
	rectangle ((int) ul->x, (int) ul->y,
		(int) (ul->x + size->x),(int) (ul->y+size->y));
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void draw_body (realpoint *position, real height, real width)
{
	realpoint size, pos;

	size.x = width;
    size.y = height;  /* size of main box */
    drawrec (position, &size);
    size.y = height/2;

	size.x = width/5;  /* size of door */
    pos.y = position->y+height/2;
    pos.x = position->x + (2.0 / 5.0 * width);
    drawrec (&pos, &size);
    size.y = height/5;

	size.x = width/5;   /* size of window */
    pos.y = position->y + height/10;
	pos.x = position->x + width/10;
    drawrec (&pos, &size);

    pos.x = position->x + (7.0 / 10.0 * width);
    drawrec (&pos, &size);
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void drawhouse (realpoint *position, realpoint *size)
{
	realpoint roofline, peak;

	roofline.x = position->x;
    roofline.y = position->y + (size->y / 3.0);
	/* roofline is left side, the position for body */
    draw_body (&roofline, size->y * 2.0 / 3.0, size->x);
	peak.y = position->y;
	peak.x = position->x + size->x/2;
    drawline (&peak, &roofline);  /* left side of roof */
	roofline.x = roofline.x + size->x;
    drawline (&peak, &roofline);  /* right side of roof */
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void WaitForKeypressWithoutTrashingGraphicsScreen()
{
	(void) getch();
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void show()
{
	realpoint housesize, housepos;

	housesize.y = getmaxy() * 0.9;
    housesize.x = 2.0 / 3.0 * housesize.y * adjustment;
	housepos.y = getmaxy() * 0.05;
	housepos.x = adjustment * housepos.y;
	drawhouse (&housepos, &housesize);
	housepos.x = getmaxx() / 2;  /* move to the right */
	housesize.x = housesize.x /2;
	housesize.y = housesize.y /2;  /* scale down */
	drawhouse (&housepos, &housesize);
	WaitForKeypressWithoutTrashingGraphicsScreen();
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

int startupgraphics()
{
	char path[100];
	int graphdriver,graphmode;
    void bgiparse(char *, char *, int *, int *);

	bgiparse(getenv("BGI"), path, &graphdriver, &graphmode);
	initgraph (&graphdriver, &graphmode, path);
	return (graphresult() == 0);
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void main()
{
	if (startupgraphics()) {
		set_adjustment();
		show();
		closegraph();
	}
	else printf("Error going into graphics mode.\n");
}
