/*****************************  LINES.C  ***********************************
*                                                                          *
*  by David Fredericks, Pathfinder Associates                              *
*                                                                          *
*  Compiled and tested with:  Borland Turbo C version 2.0                  *
*                             Microsoft C version 5.1                      *
*                             Watcom C version 7.0                         *
*                                                                          *
*  Versions 1.3  07-01-89                                                  *
*                                                                          *
*  There are NO restrictions on the use of this code,                      *
*        as long as authorship is acknowledged.                            *
*  usage:  LINES [25|28|43|50]                                             *
*                                                                          *
*  This program changes the number of text lines displayed on EGA and VGA. *
*  If no argument is given, it toggles between minimum and maximum lines.  *
*  Valid arguments for EGA are 25 and 43.  Valid arguments for VGA are 25, *
*  28, and 50.  Invalid arguments are ignored.                             *
*                                                                          *
***************************************************************************/

#include <stdlib.h>
#include <dos.h>

union   REGS    gregs;                  /* registers for int86 calls        */

#ifdef __TURBOC__
extern void interrupt (*_Int5Vector)();
#endif

                                        /* Alternative to reset EGA to     */
#define RESET_MODE     0                /* 25-line mode. Only one should   */
                                        /* be ON. See text for details.    */

                  /*- BIOS Data Area locations -*/
char far *BiosRows = (char far *)0x00400084L;   /* number of screen rows - 1*/
char far *BiosCols = (char far *)0x0040004AL;   /* number of screen columns */
char far *BiosInfo = (char far *)0x00400087L;   /* cursor emulation switch  */

#define NO_CURSOR_EMULATION 0x01        /* BIOS Info cursor emulation bit   */

#define BIOS_VIDEO  0x10
                     /*- BIOS Video Functions -*/
#define CURSORSIZE  0x01                /* set cursor scan lines            */
#define SETCURSOR   0x02                /* set cursor position              */
#define CURSORLOC   0x03                /* get current cursor position      */
#define SCRUP       0x06                /* scroll the screen up             */
#define RDCHARATTR  0x08                /* read character and attribute     */
#define GET_MODE    0x0F                /* get current video mode           */
#define CHARGEN     0x11                /* character generator interface    */
#define ALT_SELECT  0x12                /* alternate select                 */
#define DCC         0x1A                /* display combination code         */

                     /*- BIOS Video Subfunctions -*/
#define VID_CFG     0x10                /* get video configuration (EGA)    */
#define GET_DCC     0x00                /* get display combination code     */
#define PRTSC_RTN   0x20                /* select alternate PrtSc routine   */
#define CHARSET8x8  0x12                /* load ROM 8 by 8 character set    */
#define CHARSET8x14 0x11                /* load ROM 8 by 14 character set   */
#define CHARSET8x16 0x14                /* load ROM 8 by 16 character set   */

                     /*- Codes for display adapter types -*/
#define EGA         0x01                /* Enhanced Graphics Adapter        */
#define VGA         0x02                /* Video Graphics Array or MCGA     */

int CheckTextMode (void)            /* Make sure we are in text mode */
{
    gregs.h.ah = GET_MODE;          /* AH = 0x0F is get-mode service */
    int86 (BIOS_VIDEO, &gregs, &gregs);
    switch (gregs.h.al)             /* AL returns the current mode   */
    {
        case 0x00:          /* 40x25 text - color burst off */
        case 0x01:          /* 40x25 text                   */
        case 0x02:          /* 80x25 text - color burst off */
        case 0x03:          /* 80x25 text                   */
        case 0x07:          /* 80x25 text - monochrome      */
            return (1);
        default:            /* All others are graphics modes*/
            return (0);
    }
}


int DisplayType (void) /* returns 1 if EGA, 2 if VGA, otherwise 0 */
{
    gregs.h.ah = ALT_SELECT;
    gregs.h.bl = VID_CFG;
    int86(BIOS_VIDEO, &gregs, &gregs);  /* Make first test for EGA */
    if (gregs.h.bl > 3)             /* 0-3 only valid response for EGA or VGA */
        return(0);                  /* not EGA or VGA */

    gregs.h.ah = DCC;
    gregs.h.al = GET_DCC;
    int86 (BIOS_VIDEO, &gregs, &gregs); /* check for VGA */
    if (gregs.h.al != 0x1A)             /* only valid response for VGA */
        return(EGA);                    /* must be EGA */
    switch (gregs.h.bl)                 /* active adapter type */
    {
        case 0x04 :          /* EGA in color mode           */
            return(EGA);
        case 0x07 :          /* VGA with monochrome display */
        case 0x08 :          /* VGA with color display      */
            return(VGA);
        default   :          /* nothing else is of interest */
            return(0);
    }
}


void CharSetLoad(unsigned char charset) /* Loads ROM character set */
{
    gregs.h.ah = CHARGEN;
    gregs.h.al = charset;
    gregs.h.bl = 0; /* we will load the first character generator RAM block */
    int86(BIOS_VIDEO, &gregs, &gregs);
}


int CursorRow(void)    /* returns current row number of cursor */
{
    gregs.h.ah = CURSORLOC;
    gregs.h.bh = 0;
    int86(BIOS_VIDEO, &gregs, &gregs);
    return((int)gregs.h.dh);
}


void CursorLocate(int row, int col) /* moves the cursor to row and column */
{
    gregs.h.ah = SETCURSOR;
    gregs.h.bh = 0;
    gregs.h.dh = row;
    gregs.h.dl = col;
    int86(BIOS_VIDEO, &gregs, &gregs);
}


void ScrollUp(int bottom, int lines)    /* scrolls from bottom for lines */
{
    gregs.h.ah = RDCHARATTR;
    gregs.h.bh = 0;
    int86(BIOS_VIDEO, &gregs, &gregs);  /* get attribute for scroll */
    gregs.h.bh = gregs.h.ah;            /* attribute from cursor location */
    gregs.h.ah = SCRUP;
    gregs.h.al = lines;                 /* number of lines to scroll */
    gregs.h.ch = 0;                     /* top row      */
    gregs.h.cl = 0;                     /* left column  */
    gregs.h.dh = bottom;                /* bottom row   */
    gregs.h.dl = *BiosCols - 1;         /* right column */
    int86(BIOS_VIDEO, &gregs, &gregs);  /* scroll up    */
}


void SetCursorScanLines(int top, int bottom)
{
    gregs.h.ah = CURSORSIZE;
    gregs.h.ch = top;
    gregs.h.cl = bottom;
    int86(BIOS_VIDEO, &gregs, &gregs);  /* set cursor scan lines */
}


void EGAAdjust(int lines)   /* adjusts EGA cursor for line change */
{
    if (lines == 43)
    {
        *BiosInfo |= NO_CURSOR_EMULATION;  /* disable cursor emulation */
        SetCursorScanLines(6, 0);
    }
    else
    {
        *BiosInfo &= ~NO_CURSOR_EMULATION;  /* enable cursor emulation */
        SetCursorScanLines(6, 7);
    }
}


void main(int argc, char *argv[])
{
    int disptype;              /* display adapter type code        */
    int numlines;              /* number of lines in first argument*/
    int bottom;                /* bottom row of screen             */

    if (CheckTextMode() == 0)
    {
        puts ("Must be in text mode to change line count.");
        exit (2);
    }

    disptype = DisplayType();
                             /* if no arguments, toggle between line modes */
    if (argc < 2)            /* any arguments?  */
    {
        if (disptype == EGA)    /* for EGA, toggle between 43 and 25 lines */
        {
            if (*BiosRows == 42)
                numlines = 25;
            else
                numlines = 43;
        }
        else
            if (disptype == VGA) /* for VGA, toggle between 50 and 25 lines */
            {
                if (*BiosRows == 49)
                    numlines = 25;
                else
                    numlines = 50;
            }
    }
    else /* if an argument is supplied */
        numlines = atoi(argv[1]);

    if (disptype == EGA)            /* 25 and 43 valid for EGA */
    {
        if (numlines == 43)
            CharSetLoad (CHARSET8x8);
        else
            if (numlines == 25)
#if RESET_MODE
            {
                gregs.h.ah = 0;
                gregs.h.al = 0x03;
                int86(BIOS_VIDEO, &gregs, &gregs);
            }
#else 
                CharSetLoad (CHARSET8x14);
#endif
        EGAAdjust(numlines);
    }
    else
        if (disptype == VGA)        /* 25, 28, and 50 valid for VGA */
        {
            if (numlines == 50)
                CharSetLoad (CHARSET8x8);
            else
                if (numlines == 28)
                    CharSetLoad (CHARSET8x14);
                else
                    if (numlines == 25)
                        CharSetLoad (CHARSET8x16);
        }

    bottom = CursorRow();           /* where was the prompt? */
    if (bottom > *BiosRows)         /* is it below new bottom? */
    {
        CursorLocate(*BiosRows, 0); /* first move the cursor up */
        ScrollUp(bottom + 1, bottom - *BiosRows); /* then move screen up */
    }

    /* we will also switch to better PrtSc routine */
    gregs.h.ah = ALT_SELECT;
    gregs.h.bl = PRTSC_RTN;
    int86(BIOS_VIDEO, &gregs, &gregs);  /* select alternate PrtSc routine */

#ifdef __TURBOC__
    _Int5Vector = getvect(5);
#endif
}