/***************************** OURFONT.C *********************************
*    Demo of EGA font loading in C Windows Toolkit.
*    Andrew J. Chalk, Magna Carta Software, 1989.
*    Release:  The C Gazette, 1989.
*    Use freely as long as authorship is acknowledged.
*
*    Requires 1) EITHER an EGA and enhanced color or monochrome
*                display OR a VGA.
*             2) The file CURLY14.FNT in the same directory as
*                ourfont.exe at runtime.
*
*    Supports: Turbo C, Power C v. 1.1+, Watcom C v. 6.5+
**************************************************************************/

#include <conio.h>
#include <dos.h>
#include <stdio.h>


#define  TRUE   1
#define  FALSE  0


int     main(void);
int     load_fontfile(char *fspecptr, char *fontptr);
void    load_user_font(char far *p_font, int block, int bpc,
                         int char_count, int spos);
void    two_fonts(int state, int hi, int hiblock, int loblock);

char fontarray[ 16 * 256 ];

int main (void)
{
	int i, ret;

	/* INITIALIZE THE FONT BLOCK TO NULS */
	load_user_font(fontarray, 1, 16, 256, 0);

    /* Read the font from a disk file */
    /* Note file name is hard-wired in this example */

    ret = load_fontfile("curly14.fnt", fontarray);
    if (ret == EOF)
        return(EOF);

    load_user_font(fontarray, 1, 14, 256, 0);

    /* Intense chars. use Block 1 (our font), others use Block 0 */
    two_fonts(TRUE, TRUE, 1, 0);
    for(i = 0; i < 4; i++)
        printf("\nCharacter number %d\n",i);

    getch();

    return (0);
}

/*
LOAD FONTFILE --  Read font file from disk.
*/

int load_fontfile (char *fspecptr, char *fontptr)
{
    FILE *infile;
    int insize;

    if ( (infile = fopen(fspecptr, "rb")) == NULL)
    {
        puts("Error: Unable to read input file");
        getch();
        return (-1);
    }
    else
    {
        insize = fread (fontptr, 1, 14*256, infile);
        fclose (infile);
        return (insize);        /* return #items (bytes) read */
    }
}


/*
LOAD_USER_FONT -- EGA/MCGA/VGA ONLY.  Load a user-defined font
    and reset the page length.

PARMS: *p_font    -- ptr. to user table (not including terminating null);
       block      -- block to load (0-3 on EGA, 0-7 on VGA and Hercules);
       bpc        -- bytes-per-character;
       char_count -- number of characters to load (1 - 256);
       spos       -- starting position in font table.
*/
void load_user_font (char far *p_font, int block, int bpc,
                       int char_count, int spos)
{
    unsigned byte_block;
#if defined(__TURBOC__) || defined(__POWERC)
    static unsigned save_bp;
#elif defined(__WATCOMC__)
    union REGPACK r;
#endif

    byte_block = (bpc << 8) | block;

#if defined(__TURBOC__) || defined(__POWERC)
    /* Can't use intr() due to Turbo C v1.0 compiler bug.           */
    /* Note: we must do any assignments to segments prior to doing  */
    /* assignments to AX, since AX is destroyed.                    */

    _ES = FP_SEG(p_font);
    _AX = 0x1100;                   /* call function 0x11           */
    _BX = byte_block;               /* block to load                */
    _CX = char_count;               /* number of characters to load */
    _DX = spos;                     /* character offset into table  */
    save_bp  = _BP;                 /* save BP for stack addressing */
    _BP = FP_OFF(p_font);           /* load address of user font    */
    geninterrupt(0X10);
    _BP = save_bp;                  /* restore BP -- or die...      */
#elif defined(__WATCOMC__)
    r.x.ax = 0X1100;
    r.x.bx = byte_block;
    r.x.cx = char_count;
    r.x.dx = spos;
    r.x.es = FP_SEG(p_font);
    r.x.bp = FP_OFF(p_font);
    intr(0X10, &r);
#endif
}

/*
TWO_FONTS -- Set and clear the use of two fonts on the EGA.
Arguments:
    int state   -- TRUE for we're to use two fonts, else FALSE;
    int hi      -- TRUE if highlight allowed for chars. with intensity bit set.
    int hiblock -- which font block to use when intensity bit = 1
    int loblock -- which font block to use when intensity bit = 0
*/

void two_fonts (int state, int hi, int hiblock, int loblock)
{
    union REGS regs;
    int c_mode, block;

    if (state) { /* To set the use of two fonts */
        /* Selecting the desired font block is a rather amusing exercise
           in bit twiddling. The problem is as follows: we have two
           block numbers, one for the block selected by the having the
           intensity bit on, and one for characters with the intensity
           bit off.  These numbers are in the range 0-7 (VGA) or 0-3
           (EGA). The block number can thus be represented in two or
           three bits. If we say that these patterns are represented by
           these bits:
                non-intense block = 'a b c'
              & the intense block = 'x y z'
           then we must load BL with the bit pattern 'x a y z b c'

           This code does just that.
         */

        /* transform abc into a00bc */
        loblock = (loblock & 3) | ((loblock / 4) << 4);
        /* transform xyz into x0yz00 */
        if (hiblock)
            hiblock = ((hiblock / 4) << 5) | ((hiblock % 4) << 2);
        /* block = a00bc | x0yz00 */
        block = hiblock | loblock;

        regs.x.ax = 0x1103;       /* AH=11h, AL=03h: Select displayed      */
                                  /*     alphanumeric character sets       */

        regs.h.bl = (unsigned char) block;
        int86(0X10, &regs, &regs);
        if (!hi) {                /* Disable display of intensity-bit info */
            regs.x.ax = 0x1000;   /* AH=10h, AL=00h : load registers.      */
            regs.x.bx = 0x0712;   /* Load reg 12h with 07h: enables red,   */
                                  /*    green, and blue, but not intensity */
            int86(0X10, &regs, &regs); /* do it */
        }
    }
    else {                      /* To clear the use of two fonts.           */
                                /* Location 0:0x449 contains DOS's record   */
                                /* of the current video mode.               */
        c_mode = * (int far *) 0X00000449;
        regs.x.ax = c_mode;     /* now, AH=0, AL = mode.  This will invoke  */
                                /* the function  to Set Video Mode          */
        int86(0X10, &regs, &regs);
    }
}